# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Summary of a masked column. A masked column is a database column masked by a data masking request.
  class DataSafe::Models::MaskedColumnSummary
    OBJECT_TYPE_ENUM = [
      OBJECT_TYPE_TABLE = 'TABLE'.freeze,
      OBJECT_TYPE_EDITIONING_VIEW = 'EDITIONING_VIEW'.freeze,
      OBJECT_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The unique key that identifies the masked column. It's numeric and unique within a masking policy.
    # @return [String]
    attr_accessor :key

    # The unique key that identifies the parent column of the masked column.
    # @return [String]
    attr_accessor :parent_column_key

    # The OCID of the sensitive type associated with the masked column.
    # @return [String]
    attr_accessor :sensitive_type_id

    # **[Required]** The name of the schema that contains the masked column.
    # @return [String]
    attr_accessor :schema_name

    # **[Required]** The name of the object (table or editioning view) that contains the masked column.
    # @return [String]
    attr_accessor :object_name

    # **[Required]** The type of the object (table or editioning view) that contains the masked column.
    # @return [String]
    attr_reader :object_type

    # **[Required]** The name of the masked column.
    # @return [String]
    attr_accessor :column_name

    # The masking group of the masked column.
    # @return [String]
    attr_accessor :masking_column_group

    # **[Required]** The masking format used for masking the column.
    # @return [String]
    attr_accessor :masking_format_used

    # **[Required]** The total number of values masked in the column.
    # @return [Integer]
    attr_accessor :total_masked_values

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'key': :'key',
        'parent_column_key': :'parentColumnKey',
        'sensitive_type_id': :'sensitiveTypeId',
        'schema_name': :'schemaName',
        'object_name': :'objectName',
        'object_type': :'objectType',
        'column_name': :'columnName',
        'masking_column_group': :'maskingColumnGroup',
        'masking_format_used': :'maskingFormatUsed',
        'total_masked_values': :'totalMaskedValues'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'key': :'String',
        'parent_column_key': :'String',
        'sensitive_type_id': :'String',
        'schema_name': :'String',
        'object_name': :'String',
        'object_type': :'String',
        'column_name': :'String',
        'masking_column_group': :'String',
        'masking_format_used': :'String',
        'total_masked_values': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :key The value to assign to the {#key} property
    # @option attributes [String] :parent_column_key The value to assign to the {#parent_column_key} property
    # @option attributes [String] :sensitive_type_id The value to assign to the {#sensitive_type_id} property
    # @option attributes [String] :schema_name The value to assign to the {#schema_name} property
    # @option attributes [String] :object_name The value to assign to the {#object_name} property
    # @option attributes [String] :object_type The value to assign to the {#object_type} property
    # @option attributes [String] :column_name The value to assign to the {#column_name} property
    # @option attributes [String] :masking_column_group The value to assign to the {#masking_column_group} property
    # @option attributes [String] :masking_format_used The value to assign to the {#masking_format_used} property
    # @option attributes [Integer] :total_masked_values The value to assign to the {#total_masked_values} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.key = attributes[:'key'] if attributes[:'key']

      self.parent_column_key = attributes[:'parentColumnKey'] if attributes[:'parentColumnKey']

      raise 'You cannot provide both :parentColumnKey and :parent_column_key' if attributes.key?(:'parentColumnKey') && attributes.key?(:'parent_column_key')

      self.parent_column_key = attributes[:'parent_column_key'] if attributes[:'parent_column_key']

      self.sensitive_type_id = attributes[:'sensitiveTypeId'] if attributes[:'sensitiveTypeId']

      raise 'You cannot provide both :sensitiveTypeId and :sensitive_type_id' if attributes.key?(:'sensitiveTypeId') && attributes.key?(:'sensitive_type_id')

      self.sensitive_type_id = attributes[:'sensitive_type_id'] if attributes[:'sensitive_type_id']

      self.schema_name = attributes[:'schemaName'] if attributes[:'schemaName']

      raise 'You cannot provide both :schemaName and :schema_name' if attributes.key?(:'schemaName') && attributes.key?(:'schema_name')

      self.schema_name = attributes[:'schema_name'] if attributes[:'schema_name']

      self.object_name = attributes[:'objectName'] if attributes[:'objectName']

      raise 'You cannot provide both :objectName and :object_name' if attributes.key?(:'objectName') && attributes.key?(:'object_name')

      self.object_name = attributes[:'object_name'] if attributes[:'object_name']

      self.object_type = attributes[:'objectType'] if attributes[:'objectType']

      raise 'You cannot provide both :objectType and :object_type' if attributes.key?(:'objectType') && attributes.key?(:'object_type')

      self.object_type = attributes[:'object_type'] if attributes[:'object_type']

      self.column_name = attributes[:'columnName'] if attributes[:'columnName']

      raise 'You cannot provide both :columnName and :column_name' if attributes.key?(:'columnName') && attributes.key?(:'column_name')

      self.column_name = attributes[:'column_name'] if attributes[:'column_name']

      self.masking_column_group = attributes[:'maskingColumnGroup'] if attributes[:'maskingColumnGroup']

      raise 'You cannot provide both :maskingColumnGroup and :masking_column_group' if attributes.key?(:'maskingColumnGroup') && attributes.key?(:'masking_column_group')

      self.masking_column_group = attributes[:'masking_column_group'] if attributes[:'masking_column_group']

      self.masking_format_used = attributes[:'maskingFormatUsed'] if attributes[:'maskingFormatUsed']

      raise 'You cannot provide both :maskingFormatUsed and :masking_format_used' if attributes.key?(:'maskingFormatUsed') && attributes.key?(:'masking_format_used')

      self.masking_format_used = attributes[:'masking_format_used'] if attributes[:'masking_format_used']

      self.total_masked_values = attributes[:'totalMaskedValues'] if attributes[:'totalMaskedValues']

      raise 'You cannot provide both :totalMaskedValues and :total_masked_values' if attributes.key?(:'totalMaskedValues') && attributes.key?(:'total_masked_values')

      self.total_masked_values = attributes[:'total_masked_values'] if attributes[:'total_masked_values']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] object_type Object to be assigned
    def object_type=(object_type)
      # rubocop:disable Style/ConditionalAssignment
      if object_type && !OBJECT_TYPE_ENUM.include?(object_type)
        OCI.logger.debug("Unknown value for 'object_type' [" + object_type + "]. Mapping to 'OBJECT_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @object_type = OBJECT_TYPE_UNKNOWN_ENUM_VALUE
      else
        @object_type = object_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        key == other.key &&
        parent_column_key == other.parent_column_key &&
        sensitive_type_id == other.sensitive_type_id &&
        schema_name == other.schema_name &&
        object_name == other.object_name &&
        object_type == other.object_type &&
        column_name == other.column_name &&
        masking_column_group == other.masking_column_group &&
        masking_format_used == other.masking_format_used &&
        total_masked_values == other.total_masked_values
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [key, parent_column_key, sensitive_type_id, schema_name, object_name, object_type, column_name, masking_column_group, masking_format_used, total_masked_values].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
