# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Tag defaults let you specify a default tag (tagnamespace.tag=\"value\") to apply to all resource types
  # in a specified compartment. The tag default is applied at the time the resource is created. Resources
  # that exist in the compartment before you create the tag default are not tagged. The `TagDefault` object
  # specifies the tag and compartment details.
  #
  # Tag defaults are inherited by child compartments. This means that if you set a tag default on the root compartment
  # for a tenancy, all resources that are created in the tenancy are tagged. For more information about
  # using tag defaults, see [Managing Tag Defaults](https://docs.cloud.oracle.com/Content/Tagging/Tasks/managingtagdefaults.htm).
  #
  # To use any of the API operations, you must be authorized in an IAM policy. If you're not authorized,
  # talk to an administrator.
  #
  class Identity::Models::TagDefault
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The OCID of the tag default.
    # @return [String]
    attr_accessor :id

    # **[Required]** The OCID of the compartment. The tag default applies to all new resources that get created in the
    # compartment. Resources that existed before the tag default was created are not tagged.
    #
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** The OCID of the tag namespace that contains the tag definition.
    #
    # @return [String]
    attr_accessor :tag_namespace_id

    # **[Required]** The OCID of the tag definition. The tag default will always assign a default value for this tag definition.
    #
    # @return [String]
    attr_accessor :tag_definition_id

    # **[Required]** The name used in the tag definition. This field is informational in the context of the tag default.
    #
    # @return [String]
    attr_accessor :tag_definition_name

    # **[Required]** The default value for the tag definition. This will be applied to all resources created in the compartment.
    #
    # @return [String]
    attr_accessor :value

    # **[Required]** Date and time the `TagDefault` object was created, in the format defined by RFC3339.
    #
    # Example: `2016-08-25T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_created

    # The tag default's current state. After creating a `TagDefault`, make sure its `lifecycleState` is ACTIVE before using it.
    #
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** If you specify that a value is required, a value is set during resource creation (either by the
    # user creating the resource or another tag defualt). If no value is set, resource creation is
    # blocked.
    #
    # * If the `isRequired` flag is set to \"true\", the value is set during resource creation.
    # * If the `isRequired` flag is set to \"false\", the value you enter is set during resource creation.
    #
    # Example: `false`
    #
    # @return [BOOLEAN]
    attr_accessor :is_required

    # Locks associated with this resource.
    # @return [Array<OCI::Identity::Models::ResourceLock>]
    attr_accessor :locks

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'compartment_id': :'compartmentId',
        'tag_namespace_id': :'tagNamespaceId',
        'tag_definition_id': :'tagDefinitionId',
        'tag_definition_name': :'tagDefinitionName',
        'value': :'value',
        'time_created': :'timeCreated',
        'lifecycle_state': :'lifecycleState',
        'is_required': :'isRequired',
        'locks': :'locks'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'compartment_id': :'String',
        'tag_namespace_id': :'String',
        'tag_definition_id': :'String',
        'tag_definition_name': :'String',
        'value': :'String',
        'time_created': :'DateTime',
        'lifecycle_state': :'String',
        'is_required': :'BOOLEAN',
        'locks': :'Array<OCI::Identity::Models::ResourceLock>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :tag_namespace_id The value to assign to the {#tag_namespace_id} property
    # @option attributes [String] :tag_definition_id The value to assign to the {#tag_definition_id} property
    # @option attributes [String] :tag_definition_name The value to assign to the {#tag_definition_name} property
    # @option attributes [String] :value The value to assign to the {#value} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [BOOLEAN] :is_required The value to assign to the {#is_required} property
    # @option attributes [Array<OCI::Identity::Models::ResourceLock>] :locks The value to assign to the {#locks} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.tag_namespace_id = attributes[:'tagNamespaceId'] if attributes[:'tagNamespaceId']

      raise 'You cannot provide both :tagNamespaceId and :tag_namespace_id' if attributes.key?(:'tagNamespaceId') && attributes.key?(:'tag_namespace_id')

      self.tag_namespace_id = attributes[:'tag_namespace_id'] if attributes[:'tag_namespace_id']

      self.tag_definition_id = attributes[:'tagDefinitionId'] if attributes[:'tagDefinitionId']

      raise 'You cannot provide both :tagDefinitionId and :tag_definition_id' if attributes.key?(:'tagDefinitionId') && attributes.key?(:'tag_definition_id')

      self.tag_definition_id = attributes[:'tag_definition_id'] if attributes[:'tag_definition_id']

      self.tag_definition_name = attributes[:'tagDefinitionName'] if attributes[:'tagDefinitionName']

      raise 'You cannot provide both :tagDefinitionName and :tag_definition_name' if attributes.key?(:'tagDefinitionName') && attributes.key?(:'tag_definition_name')

      self.tag_definition_name = attributes[:'tag_definition_name'] if attributes[:'tag_definition_name']

      self.value = attributes[:'value'] if attributes[:'value']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.is_required = attributes[:'isRequired'] unless attributes[:'isRequired'].nil?

      raise 'You cannot provide both :isRequired and :is_required' if attributes.key?(:'isRequired') && attributes.key?(:'is_required')

      self.is_required = attributes[:'is_required'] unless attributes[:'is_required'].nil?

      self.locks = attributes[:'locks'] if attributes[:'locks']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        compartment_id == other.compartment_id &&
        tag_namespace_id == other.tag_namespace_id &&
        tag_definition_id == other.tag_definition_id &&
        tag_definition_name == other.tag_definition_name &&
        value == other.value &&
        time_created == other.time_created &&
        lifecycle_state == other.lifecycle_state &&
        is_required == other.is_required &&
        locks == other.locks
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, compartment_id, tag_namespace_id, tag_definition_id, tag_definition_name, value, time_created, lifecycle_state, is_required, locks].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
